/*********************************************************************************/
/*																				 */
/*						Routinen zur I2C-Bus Steuerung 							 */
/*																				 */
/*********************************************************************************/


#include <reg167.h>
#include <stdio.h>
#include <string.h>


// 10 ms Timeout fuer Bus Fehler
#define Period 20000		

// Hardwareadresse des I2C EEProms
#define EEprom 0xA0

#define ulong unsigned long
#define uint unsigned int
#define uchar unsigned char

// Definition der Port-Pins fuer SCL, SDA und deren Richtungen 
sbit SCL_Direction = DP7^5;
sbit SDA_Direction = DP7^4;
sbit SCL = P7^5; 
sbit SDA = P7^4;


#ifndef i2c
#define i2c

/*********************************************************************************/
/*					  		  	Definitionen 					 		 		 */
/*********************************************************************************/
/*
// 10 ms Timeout fuer Bus Fehler
#define Period 20000		

// Hardwareadresse des I2C EEProms
#define EEprom 0xA0

#define ulong unsigned long
#define uint unsigned int
#define uchar unsigned char

// Definition der Port-Pins fuer SCL, SDA und deren Richtungen 
sbit SCL_Direction = DP7^5;
sbit SDA_Direction = DP7^4;
sbit SCL = P7^5; 
sbit SDA = P7^4;
*/
/*
void Delay(uint count);
void I2C_CheckClock(void);
uchar Check_SCL(void);
uchar I2C_Init(void);
void I2C_Start(void);
uchar I2C_Write(uchar input_byte);
uchar I2C_Read(uchar ack);
uchar I2C_Stop();
uchar I2C_CheckWrite(uchar Device_Adresse);
uchar I2C_Write_Data(uchar Device_Adresse, int Sub_Adresse, uchar *Buffer,uchar Count);
uchar I2C_Read_Data(uchar Device_Adresse, int Sub_Adresse, uchar *Buffer, uchar Count);
*/
uint time_out;



//---------------------------------------------------------------------------------
//  Delay(); kleine Warteschleife
//  Bleibt in der Schleife bis 'count' null wird 
//  Input:	        1 count = 400ns
//  Return:			None
//---------------------------------------------------------------------------------

void Delay(uint count)
{
   while (count--);
}



//---------------------------------------------------------------------------------
//  I2C_CheckClock(); wartet bis das Slave Device die Clock Leitung freigibt (Wait Condition)
//					  des I2C Busses
//---------------------------------------------------------------------------------

void I2C_CheckClock(void)
{
   SCL_Direction = 0;			// SCL auf Input
   while (!SCL);           		// warten bis SCL freigegeben wird
   SCL = 1;
   SCL_Direction = 1; 			// SCL auf Output
}


//---------------------------------------------------------------------------------
//  uchar Check_SCL(); ueperprueft ob die SCL Leitung von allen Slaves freigelassen wurde
//		Rueckgabewert:	0 -	wenn SCL ist OK, 		1 -	wenn ein Fehler vorliegt
//---------------------------------------------------------------------------------

uchar Check_SCL(void)
{
   time_out = Period;
   SCL_Direction = 0;      		// SCL ist Input
   while (time_out--)	
   {  if (SCL)      			// wenn SCL auf nicht mehr low gezogen wird
	  { SCL = 1;          		// SCL auf High setzen
	     SCL_Direction = 1;    	// SCL ist Output
 	     return (0);
	   }
    }
	return (1);   				// Fehler, SCL wird auf Low gezogen
}




//---------------------------------------------------------------------------------
//  I2C_Start();  Generiert die Startbedingung am I2C-Bus
//---------------------------------------------------------------------------------

void I2C_Start()
{
   SDA = 1;        			// Scherstellen dass SDA und SCL Hig sind
   SCL = 1;

   SDA_Direction = 1;		// SDA und SCL auf Output
   SCL_Direction = 1;

   Delay(5);

   SDA = 0;        			// SDA wird zuerst Low
   Delay(10);
   SCL = 0;  				// dann folgt nach einer kurzen Pause auch SCL
}


//---------------------------------------------------------------------------------
//  I2C_Write (Daten);  Schreibt die Daten auf den I2C Bus
//		Rueckgabewert: Acknowledge Bit  (0=OK, 1=Fehler)
//---------------------------------------------------------------------------------

uchar I2C_Write(uchar Input_Byte)
{
   uchar mask,i;
  
   mask = 0x80;                   	// Maske = 1000 0000
   SDA_Direction = 1;				// SDA auf Output

   for (i=0; i<8; i++)				// Sende ein Byte
   { if (mask & Input_Byte) 		// Bits rauspopeln
	 SDA = 1;
	 else SDA = 0;
	 mask = mask >> 1;				// schiebe eins nach rechts fuer naechstes Bit

     Delay(0);
     I2C_CheckClock();       		// warten bis Slave die SCL Leitung freibibt
     Delay (1);

     SCL = 0;                     	// Clock auf Low
	 Delay(3);
   }

   SDA_Direction = 0;      			// SDA auf Input (fuer Acknowledge Bit)
   Delay(3);
   SCL = 1;							// 9ten Clock Puls generieren
   Delay (0);
   mask = SDA;						// Lese Acknowledge Bit
   Delay(1);
   SCL = 0;                
   Delay(6);						

   return (mask);          			// Acknowledge Bit zurueckgeben
}	
						     			

//---------------------------------------------------------------------------------
//  uchar I2C_Write (Ack);  Liest Daten vom I2C-Bus und generiert Ack-Bit (wie uebergeben)
//		Rueckgabewert: empfangenes Byte
//---------------------------------------------------------------------------------
				
uchar I2C_Read(uchar ack)
{
   uchar mask,i,rec_data;

   rec_data = 0;	
   mask = 0x80;
   SDA_Direction = 0;				// SDA auf Output
	
   for (i=0; i<8; i++)				// Bits reinpopeln
   { Delay(2);
     I2C_CheckClock();       		// warten bis Slave die SCL Leitung freigibt
     Delay(0);
     if (SDA)                    	// waehrend Clock High ist wird SDA eingelesen
     rec_data |= mask;
     mask = mask >> 1;
     SCL = 0;                    	// Clock wird Low
   } 
   if (ack) SDA = 1;             	// SDA Acknowledge setzen, solange SDA noch auf Input gesetzt ist
   else SDA = 0;					
       
   SDA_Direction = 1;				// SDA auf Output
   Delay(1);
   SCL = 1;    	  					// Clockpuls ausgeben
   Delay(4);
   SCL = 0;                
   Delay(6);						
   return (rec_data);
}	


//---------------------------------------------------------------------------------
//  uchar I2C_Stopp();  Generiert die Stopbedingung am I2C-Bus
//---------------------------------------------------------------------------------
				
uchar I2C_Stop()
{
   time_out = Period;
   SDA_Direction =0;        		// SDA auf Input
   while (time_out--)
   { if (!SDA)                   	// Erzeuge Clock Pulse solange SDA auf Low gezogen wird
	 { SCL = 1;                		// oder max. 2000mal (=Period)
	   Delay(6);		
	   SCL = 0;
	   Delay(6);
	 }
	 else                 			// ueberpruefe SCL
     { SDA  = 0;					// SDA auf Low
	   SDA_Direction = 1;       	// SDA auf Output
	   if (Check_SCL())  return (1);     // wenn SCL auf Low gehalten wird, wird ein Fehler ausgegeben
       Delay(10);
	   SDA = 1;						// SDA aendert sich waehrend SCL High (Stoppbedingung)
	   return (0);
	 }
   }
   return (1);   	       			// Timeout Fehler, SDA wird auf Low gehalten
}


//---------------------------------------------------------------------------------
//  I2C_CheckWrite();  Sendet ein Lesekomando und ueberpueft das Acknowledge Bit
//  	Rueckgabewerte: 1 - wenn Device vorhanden;   0 - Wenn Device nicht vorhanden
//---------------------------------------------------------------------------------

uchar I2C_CheckWrite(uchar Device_Adresse)
{
   uchar Ack;
   I2C_Start();
   Ack = I2C_Write(Device_Adresse + 1); 		// sende Leseadresse und empfange Acknowledge Bit
   return (!Ack);
} 


//---------------------------------------------------------------------------------
//  I2C_Write_Data (Adresse, Sub_Adresse, *Buffer, Count)
//		Uebergabeparameter:		Device_Adresse - Hardware Adresse des I2C Bausteins
//								Sub_Adresse - Softwareadresse des I2C Bausteins (wenn nicht benoetigt = -1)
//								*Buffer - Speicheradresse der zu sendenden Daten
//								Count - Anzahl der zu sendenden Bytes
//		Rueckgabeparameter:		0 - Schreibvorgang OK,      1 - Fehler (Acknowledge Bit)
//---------------------------------------------------------------------------------

uchar I2C_Write_Data(uchar Device_Adresse, int Sub_Adresse, uchar *Buffer, uchar Count)
{
   uchar i;
   I2C_Start();    					// Startbedingung
   if (I2C_Write(Device_Adresse))   // Wenn kein Acnowledge Bit zurckgeliefert wird
   { I2C_Stop();					// wird wieder die Stoppbedingung ausgegeben
	 return(1);	
   }
   if (Sub_Adresse != -1)           // Wenn Unteradressen verwendet werden sollen
   { if (I2C_Write(Sub_Adresse))  	// wird das Acknowledge Bit der Unteradresse ueberprueft
     { I2C_Stop();					// und bei einem Fehler wird die Stoppbedingung ausgegeben
	   return(1);
	 }
   }
   for (i = 0; i < Count; i++)		// Datenbytes werden gesendet
   { if (I2C_Write(*Buffer))           
	 { I2C_Stop();
	   return(1);
	 }
   }
   I2C_Stop();                     	// Stoppbedingung ausgeben
   return(0);
}		
 

//---------------------------------------------------------------------------------
//  I2C_Read_Data (Adresse, Sub_Adresse, *Buffer, Count)
//		Uebergabeparameter:		Device_Adresse - Hardware Adresse des I2C Bausteins
//								Sub_Adresse - Softwareadresse des I2C Bausteins (wenn nicht benoetigt = -1)
//								*Buffer - Speicheradresse der empfangenen Daten
//								Count - Anzahl der zu empfangenen Bytes
//		Rueckgabeparameter:		0 - Lesevorgang OK,      1 - Fehler (Acknowledge Bit)
//---------------------------------------------------------------------------------
				
uchar I2C_Read_Data(uchar Device_Adresse, int Sub_Adresse, uchar *Buffer, uchar Count)
{
   uchar i;
   I2C_Start();           	 			// Startbedingung
   if (I2C_Write(Device_Adresse)) 		// Wenn kein Acknowledge Bit zurueckgegeben wird
   { I2C_Stop();						// wird wieder die Stoppbedingung ausgegeben
	 return(1);
   }
   if (Sub_Adresse != -1)             	// WEnn Unteradressen verwendet werden sollen
   { if (I2C_Write(Sub_Adresse))		// wird das Acknowledge Bit der Unteradresse ueberprueft
     { I2C_Stop();						// und bei einem Fehler wird die Stoppbedingung ausgegeben
	   return(1);
	 }
   }
   I2C_Start();                     	// Startbedingung ausgeben
   if (I2C_Write(Device_Adresse + 1))  	// Acknowledge Bit der Leseadresse ueberpruefen
   { I2C_Stop();						// bei Fehler wieder die Stoppbedingung ausgeben
	 return(1);
   }
   for (i = 0; i < Count - 1; i++)   	// angegebene Anzahl der Datenbytes werden gelesen
   { *Buffer = I2C_Read(0);
	 Buffer++;
   }
   *Buffer = I2C_Read(1);     			// Das letzte Byte wird mit einem High Acknowledge eingelesen
   I2C_Stop();                      	// Stoppbedingung ausgeben
   return(0);
}





//---------------------------------------------------------------------------------
//  I2C_Init();  Initialisiert das I2C Interface
//		Rueckgabewert: Acknowledge Bit  (0=OK, 1=Fehler)
//---------------------------------------------------------------------------------

unsigned char I2C_Init()
{
   SDA_Direction = 0;    						// SDA und SCL auf Input
   SCL_Direction = 0;

   if (!SDA)  if (I2C_Stop())  return (1); 		// wenn SDA oder SCL low gehalten werden wird ein Fehler zurueckgegeben

   if (!SCL)  if (I2C_Stop())  return (1);
	
   return (0);
}
		


#endif
