#include <reg167.h>
#include <stdio.h>
#include <string.h>
#include "i2c.h"

// 10 ms Timeout fuer Bus Fehler
#define Period 20000		

// Hardwareadresse des I2C EEProms
#define EEprom 0xA0

#define ulong unsigned long
#define uint unsigned int
#define uchar unsigned char

// Definition der Port-Pins fuer SCL, SDA und deren Richtungen 
sbit SCL_Direction = DP7^5;
sbit SDA_Direction = DP7^4;
sbit SCL = P7^5; 
sbit SDA = P7^4;

sbit P310 = P3^10; 	// Namenvergabe fuer Portpins	
sbit DP310 = DP3^10;// Namenvergabe fuer Richtung

//uint time_out;




//Prototypen
unsigned char _hex2dec( unsigned char hex );
unsigned char _dec2hex( unsigned char dec );
void rtc_setTime( unsigned char hour, unsigned char min, unsigned char sec, unsigned char hund );
void rtc_getTime( unsigned char * hour, unsigned char * min, unsigned char * sec, unsigned char * hund );
void rtc_WriteRAM( unsigned char dptr, unsigned char dat );
void rtc_ReadRAM( unsigned char dptr, unsigned char * dat );
void rtc_getDate( unsigned char * day, unsigned char * month, unsigned char * weekday );
void rtc_setDate( unsigned char day, unsigned char month, unsigned char weekday, unsigned char schaltjahr );



unsigned char _hex2dec( unsigned char hex )
{
   int hex2=(int)hex;
   int ret=0;

   ret +=((int)(hex2/100)*256);
   hex2-=((int)(hex2/100)*100);
   ret +=((int)(hex2/10)*16);
   hex2-=((int)(hex2/10)*10);
   ret +=((int)(hex2));

   return ret;
}

unsigned char _dec2hex( unsigned char dec )
{
   int dec2=(int)dec;
   int ret=0;

   ret +=((int)(dec2/256)*100);
   dec2-=((int)(dec2/256)*256);
   ret +=((int)(dec2/16)*10);
   dec2-=((int)(dec2/16)*16);
   ret +=((int)(dec2));

   return ret;
}

/*
unsigned char rtc_exists()
{
   return i2c_exists( 0xA0 );
}
*/

void rtc_setTime( unsigned char hour, unsigned char min, unsigned char sec, unsigned char hund )
{
   I2C_Start();
   I2C_Write( 0xA0 );      // Slave Adress
   I2C_Write( 0x01 );      // set pointer to control status
   I2C_Write( _hex2dec(hund) );
                           // to hundreth of a second

   I2C_Start();
   I2C_Write( 0xA0 );      // Slave Adress
   I2C_Write( 0x02 );      // set pointer to control status
   I2C_Write( _hex2dec(sec) );
                           // to hundreth of a second

   I2C_Start();
   I2C_Write( 0xA0 );      // Slave Adress
   I2C_Write( 0x03 );      // set pointer to control status
   I2C_Write( _hex2dec(min) );
                           // to hundreth of a second

   I2C_Start();
   I2C_Write( 0xA0 );      // Slave Adress
   I2C_Write( 0x04 );      // set pointer to control status
   I2C_Write( _hex2dec(hour) );
                           // to hundreth of a second

   I2C_Stop();
}

void rtc_getTime( unsigned char * hour, unsigned char * min,
                  unsigned char * sec, unsigned char * hund )
{

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x01 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable read mode
   *hund = _dec2hex( I2C_Read(1) );

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x02 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable read mode
   *sec = _dec2hex( I2C_Read(1) );

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x03 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable read mode
   *min  = _dec2hex( I2C_Read(1) );

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x04 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable read mode
   *hour = _dec2hex( I2C_Read(1) & 0x3F );

   I2C_Stop();
}

void rtc_WriteRAM( unsigned char dptr, unsigned char dat )
{
   I2C_Start();
   I2C_Write( 0xA0 );
   I2C_Write( dptr );
   I2C_Write( dat );
   I2C_Stop();
}

void rtc_ReadRAM( unsigned char dptr, unsigned char * dat )
{
   I2C_Start();
   I2C_Write( 0xA0 );
   I2C_Write( dptr );
   I2C_Start();
   I2C_Write( 0xA1 );
   *dat = I2C_Read( 1 );
   I2C_Stop();
}

void rtc_getDate( unsigned char * day, unsigned char * month, unsigned char * weekday )
{
   unsigned char d1;

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x05 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable Read mode
   d1 = I2C_Read(1);
   *day = (((d1 & 0x30)>>4)*10) + (d1 & 0x0F);

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x06 );      // set data pointer
   I2C_Start();
   I2C_Write(0xA1);        // enable read mode
   d1 = I2C_Read(1);
   *month = (((d1 & 0x10)>>4)*10) + (d1 & 0x0F);
   *weekday = (d1 & 0xE0)>>5;

   I2C_Stop();            
}

void rtc_setDate( unsigned char day, unsigned char month, unsigned char weekday, unsigned char schaltjahr )
{
   unsigned char d1;

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x05 );      // set data pointer
   I2C_Write( (((int)(day/10))<<4) +
              (schaltjahr<<6) +
                     (day%10) );

   I2C_Start();
   I2C_Write( 0xA0 );      // Address
   I2C_Write( 0x06 );      // set data pointer
   I2C_Write( (((int)(month/10))<<4) +
                        (month%10) +
                        (weekday<<5) );

   I2C_Stop();            
      
}





void main()
{
   unsigned char hs, sec, min, hour;
   char temp[21];

//**** Initialisation Serial Port ****//
   P310=1;         	// set Port 3.10 output latch TxD 
   DP310=1;        	// set Port 3.10 direction control TxD out      
   S0TIC=0x80;     	// set transmit interrupt flag
   S0BG= 0x33;     	// set baudrate 9600 @ 16 MC CPU clock
   S0CON=0x8011;   	// set serial mode 

   rtc_setTime( 12, 10, 30, 50 );
  
   while(1)
   {
        rtc_getTime( &hour, &min, &sec, &hs );        

        printf("%02d:%02d:%02d:%02d\n", (int)hour, (int)min, (int)sec, (int)hs) ;
        Delay(30000);
   }
}
